"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.YfmEditor = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importDefault(require("react"));
const YfmEditorView_1 = require("./YfmEditorView");
const useYfmEditor_1 = require("./useYfmEditor");
/**
 * @deprecated
 * The `YfmEditor` component will be removed soon.
 * Use `useYfmEditor` hook and `YfmEditorView` component instead.
 */
exports.YfmEditor = react_1.default.forwardRef(function YfmEditor({ className, splitMode, autofocus, initialContent, initialEditorType = 'wysiwyg', initialMenuVisible: initialToolbarVisible = true, initialSplitModeEnabled = false, markupMenubarData: markupToolbarData, wysiwygMenubarData: wysiwygToolbarData, wysiwygExtraExtensions, wysiwygAllowHTML, wysiwygLinkify, wysiwygLinkifyTlds, wysiwygAttrs, wysiwygBreaks = true, extensionOptions, prepareRawMarkup, onMarkupChange, onMenuBarAction: onToolbarAction, onSplitModeEnabledChange, renderPreview, onEditorTypeChange, onMenuVisibleChange: onToolbarVisibleChange, onCancel, onSubmit, onFileUpload, needToSetDimensionsForUploadedImages, settingsVisible, toaster, wysiwygHiddenActionsConfig, markupHiddenActionsConfig, stickyToolbar = true, }, ref) {
    const editor = (0, useYfmEditor_1.useYfmEditor)({
        splitMode,
        renderPreview,
        initialMarkup: initialContent,
        initialEditorType,
        initialToolbarVisible,
        initialSplitModeEnabled,
        allowHTML: wysiwygAllowHTML,
        linkify: wysiwygLinkify,
        linkifyTlds: wysiwygLinkifyTlds,
        breaks: wysiwygBreaks,
        attrs: wysiwygAttrs,
        extraExtensions: wysiwygExtraExtensions,
        extensionOptions,
        prepareRawMarkup,
        fileUploadHandler: onFileUpload,
        needToSetDimensionsForUploadedImages,
    });
    const viewRef = react_1.default.useRef(null);
    const self = react_1.default.useMemo(() => {
        const editorRef = editor;
        editorRef.domElem = () => viewRef.current;
        return editor;
    }, [editor]);
    react_1.default.useImperativeHandle(ref, () => self, [self]);
    react_1.default.useLayoutEffect(() => {
        if (!onSplitModeEnabledChange)
            return;
        const cb = ({ splitModeEnabled }) => {
            onSplitModeEnabledChange(splitModeEnabled);
        };
        self.on('change-split-mode-enabled', cb);
        return () => {
            self.off('change-split-mode-enabled', cb);
        };
    });
    react_1.default.useLayoutEffect(() => {
        if (!onEditorTypeChange)
            return;
        const cb = ({ type }) => {
            onEditorTypeChange(type);
        };
        self.on('change-editor-type', cb);
        return () => {
            self.off('change-editor-type', cb);
        };
    }, [onEditorTypeChange, self]);
    react_1.default.useLayoutEffect(() => {
        if (!onMarkupChange)
            return;
        const cb = () => {
            onMarkupChange(self, self.currentType);
        };
        self.on('change', cb);
        return () => {
            self.off('change', cb);
        };
    }, [onMarkupChange, self]);
    react_1.default.useLayoutEffect(() => {
        if (!onCancel)
            return;
        const cb = () => {
            onCancel(self);
        };
        self.on('cancel', cb);
        return () => {
            self.off('cancel', cb);
        };
    }, [onCancel, self]);
    react_1.default.useLayoutEffect(() => {
        if (!onSubmit)
            return;
        const cb = () => {
            onSubmit(self);
        };
        self.on('submit', cb);
        return () => {
            self.off('submit', cb);
        };
    }, [onSubmit, self]);
    react_1.default.useLayoutEffect(() => {
        if (!onToolbarAction)
            return;
        const cb = ({ id, editorType }) => {
            onToolbarAction({ action: id, editorType });
        };
        self.on('toolbar-action', cb);
        return () => {
            self.off('toolbar-action', cb);
        };
    }, [onToolbarAction, self]);
    react_1.default.useLayoutEffect(() => {
        if (!onToolbarVisibleChange)
            return;
        const cb = ({ visible }) => {
            onToolbarVisibleChange(visible);
        };
        self.on('change-toolbar-visibility', cb);
        return () => {
            self.off('change-toolbar-visibility', cb);
        };
    }, [onToolbarVisibleChange, self]);
    return (react_1.default.createElement(YfmEditorView_1.YfmEditorView, { markupHiddenActionsConfig: markupHiddenActionsConfig, wysiwygHiddenActionsConfig: wysiwygHiddenActionsConfig, ref: viewRef, editor: editor, autofocus: autofocus, settingsVisible: settingsVisible, markupToolbarConfig: markupToolbarData, wysiwygToolbarConfig: wysiwygToolbarData, className: className, toaster: toaster, stickyToolbar: stickyToolbar }));
});
