"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.withLogger = exports.config = void 0;
const codemirror_1 = require("codemirror");
require("codemirror/addon/display/placeholder");
require("codemirror/addon/edit/continuelist");
// import 'codemirror/addon/fold/foldcode';
// import 'codemirror/addon/fold/foldgutter';
// import 'codemirror/addon/fold/foldgutter.css';
require("codemirror/addon/fold/markdown-fold");
// import 'codemirror/addon/selection/active-line';
require("codemirror/mode/markdown/markdown");
const logger_1 = require("../logger");
const commands_1 = require("../markup/commands");
const shortcuts_1 = require("../shortcuts");
const action_names_1 = require("./config/action-names");
require("codemirror/lib/codemirror.css");
exports.config = {
    lineNumbers: false,
    lineWrapping: true,
    // foldGutter: true,
    mode: 'text/markdown',
    viewportMargin: Infinity,
    // gutters: ['CodeMirror-linenumbers', 'CodeMirror-foldgutter'],
    extraKeys: {
        Enter: 'newlineAndIndentContinueMarkdownList',
        [shortcuts_1.formatter.toCM(shortcuts_1.Action.Bold)]: withLogger(action_names_1.ActionName.bold, commands_1.wrapToBold),
        [shortcuts_1.formatter.toCM(shortcuts_1.Action.Italic)]: withLogger(action_names_1.ActionName.italic, commands_1.wrapToItalic),
        [shortcuts_1.formatter.toCM(shortcuts_1.Action.Strike)]: withLogger(action_names_1.ActionName.strike, commands_1.wrapToStrike),
        [shortcuts_1.formatter.toCM(shortcuts_1.Action.Underline)]: withLogger(action_names_1.ActionName.underline, commands_1.wrapToUnderline),
        [shortcuts_1.formatter.toCM(shortcuts_1.Action.Link)]: withLogger(action_names_1.ActionName.link, (0, commands_1.insertLink)({ url: '' })),
        [shortcuts_1.formatter.toCM(shortcuts_1.Action.Heading1)]: withLogger(action_names_1.ActionName.heading1, commands_1.toH1),
        [shortcuts_1.formatter.toCM(shortcuts_1.Action.Heading2)]: withLogger(action_names_1.ActionName.heading2, commands_1.toH2),
        [shortcuts_1.formatter.toCM(shortcuts_1.Action.Heading3)]: withLogger(action_names_1.ActionName.heading3, commands_1.toH3),
        [shortcuts_1.formatter.toCM(shortcuts_1.Action.Heading4)]: withLogger(action_names_1.ActionName.heading4, commands_1.toH4),
        [shortcuts_1.formatter.toCM(shortcuts_1.Action.Heading5)]: withLogger(action_names_1.ActionName.heading5, commands_1.toH5),
        [shortcuts_1.formatter.toCM(shortcuts_1.Action.Heading6)]: withLogger(action_names_1.ActionName.heading6, commands_1.toH6),
        [shortcuts_1.formatter.toCM(shortcuts_1.Action.Code)]: withLogger(action_names_1.ActionName.code_inline, commands_1.toInlineCode),
        [shortcuts_1.formatter.toCM(shortcuts_1.Action.CodeBlock)]: withLogger(action_names_1.ActionName.code_block, commands_1.toCodeBlock),
        [shortcuts_1.formatter.toCM(shortcuts_1.Action.Cut)]: withLogger(action_names_1.ActionName.yfm_cut, commands_1.wrapToCut),
        [shortcuts_1.formatter.toCM(shortcuts_1.Action.Note)]: withLogger(action_names_1.ActionName.yfm_note, commands_1.wrapToNote),
    },
    // styleActiveLine: true,
    addModeClass: true,
};
function withLogger(action, command) {
    return (...args) => {
        const res = command(...args);
        if (res !== codemirror_1.Pass) {
            logger_1.logger.action({ mode: 'markup', source: 'keymap', action });
        }
        return res;
    };
}
exports.withLogger = withLogger;
