"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.renderUploadWidget = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importDefault(require("react"));
const icons_1 = require("@gravity-ui/icons");
const uikit_1 = require("@gravity-ui/uikit");
const react_dom_1 = require("react-dom");
const classname_1 = require("../../classname");
const icons_2 = require("../config/icons");
function renderUploadWidget(container, props) {
    return () => (0, react_dom_1.createPortal)(react_1.default.createElement(UploadWidget, Object.assign({}, props)), container);
}
exports.renderUploadWidget = renderUploadWidget;
const cnWidget = (0, classname_1.cn)('upload-widget');
function UploadWidget({ files, onReUploadClick, onCloseClick }) {
    if (!files.length)
        return null;
    const showCloseButton = files.every((f) => f.status === 'error');
    return (react_1.default.createElement("div", { className: cnWidget() },
        react_1.default.createElement("div", { className: cnWidget('labels') },
            "Uploading files:",
            files.map((file) => (react_1.default.createElement(UploadLabel, Object.assign({}, file, { key: file.fileName, onReUploadClick: () => onReUploadClick(file) }))))),
        showCloseButton && (react_1.default.createElement(uikit_1.Button, { size: "xs", view: "flat-secondary", onClick: onCloseClick, className: cnWidget('close-button') },
            react_1.default.createElement(uikit_1.Icon, { data: icons_1.Xmark })))));
}
const cnLabel = (0, classname_1.cn)('upload-label');
function UploadLabel({ fileName, fileType, status, errorText, onReUploadClick }) {
    const icon = fileType === 'image' ? icons_2.icons.image : icons_2.icons.file;
    if (status === 'uploading') {
        return (react_1.default.createElement(uikit_1.Label, { size: "xs", theme: "info", icon: react_1.default.createElement(uikit_1.Icon, Object.assign({}, icon)), className: cnLabel({ status }) },
            react_1.default.createElement("div", { className: cnLabel('content') },
                react_1.default.createElement("span", { title: fileName, className: cnLabel('filename') }, fileName),
                react_1.default.createElement(uikit_1.Spin, { size: "xs" }))));
    }
    if (status === 'success') {
        return (react_1.default.createElement(uikit_1.Label, { size: "xs", theme: "success", icon: react_1.default.createElement(uikit_1.Icon, Object.assign({}, icon)), className: cnLabel({ status }) },
            react_1.default.createElement("div", { className: cnLabel('content') },
                react_1.default.createElement("span", { title: fileName, className: cnLabel('filename') }, fileName))));
    }
    if (status === 'error') {
        return (react_1.default.createElement(uikit_1.Tooltip, { content: errorText },
            react_1.default.createElement(uikit_1.Label, { size: "xs", interactive: true, theme: "danger", icon: react_1.default.createElement(uikit_1.Icon, Object.assign({}, icon)), onClick: onReUploadClick, className: cnLabel({ status }) },
                react_1.default.createElement("div", { className: cnLabel('content') },
                    react_1.default.createElement("span", { className: cnLabel('filename') }, fileName),
                    react_1.default.createElement(uikit_1.Icon, { data: icons_1.ArrowsRotateRight, size: 16 })))));
    }
    return null;
}
