"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ToolbarFile = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importDefault(require("react"));
const uikit_1 = require("@gravity-ui/uikit");
const FileForm_1 = require("../../../forms/FileForm");
const forms_1 = require("../../../i18n/forms");
const menubar_1 = require("../../../i18n/menubar");
const hooks_1 = require("../../../react-utils/hooks");
const toaster_1 = require("../../../react-utils/toaster");
const toolbar_1 = require("../../../toolbar");
const upload_1 = require("../../../utils/upload");
const icons_1 = require("../../config/icons");
const placement = ['bottom-start', 'top-start', 'bottom-end', 'top-end'];
const ToolbarFile = ({ className, active, enable, onSubmit, focus, onClick, uploadHandler, onSuccessUpload, }) => {
    var _a;
    const toaster = (0, toaster_1.useToaster)();
    const buttonRef = react_1.default.useRef(null);
    const [open, , hide, toggleOpen] = (0, hooks_1.useBooleanState)(false);
    const [loading, showLoading, hideLoading] = (0, hooks_1.useBooleanState)(false);
    const popupOpen = enable && open;
    const shouldForceHide = open && !popupOpen;
    react_1.default.useLayoutEffect(() => {
        if (shouldForceHide) {
            hide();
        }
    }, [hide, shouldForceHide]);
    return (react_1.default.createElement(react_1.default.Fragment, null,
        react_1.default.createElement(uikit_1.Tooltip, { disabled: popupOpen, content: (0, menubar_1.i18n)('file'), openDelay: toolbar_1.ToolbarTooltipDelay.Open, closeDelay: toolbar_1.ToolbarTooltipDelay.Close },
            react_1.default.createElement(uikit_1.Button, { size: "m", ref: buttonRef, view: active || popupOpen ? 'normal' : 'flat', selected: active, disabled: !enable, className: className, onClick: toggleOpen },
                react_1.default.createElement(uikit_1.Icon, { data: icons_1.icons.file.data, size: (_a = icons_1.icons.file.size) !== null && _a !== void 0 ? _a : 16 }))),
        react_1.default.createElement(uikit_1.Popup, { anchorRef: buttonRef, open: popupOpen, onClose: hide, placement: placement },
            react_1.default.createElement(FileForm_1.FileForm, { autoFocus: true, onAttach: uploadHandler &&
                    ((files) => {
                        showLoading();
                        (0, upload_1.batchUploadFiles)(files, uploadHandler).then((res) => {
                            hideLoading();
                            hide();
                            onSuccessUpload === null || onSuccessUpload === void 0 ? void 0 : onSuccessUpload(res);
                        }, (error) => {
                            hideLoading();
                            toaster.add({
                                theme: 'danger',
                                name: 'toolbar_file_upload',
                                title: (0, forms_1.i18n)('file_upload_failed'),
                                content: String(error),
                            });
                        });
                    }), onCancel: hide, onSubmit: (data) => {
                    hide();
                    focus();
                    onSubmit(data);
                    onClick === null || onClick === void 0 ? void 0 : onClick('addFile');
                }, loading: loading }))));
};
exports.ToolbarFile = ToolbarFile;
