"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ToolbarLink = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importDefault(require("react"));
const uikit_1 = require("@gravity-ui/uikit");
const LinkForm_1 = require("../../../forms/LinkForm");
const menubar_1 = require("../../../i18n/menubar");
const hooks_1 = require("../../../react-utils/hooks");
const toolbar_1 = require("../../../toolbar");
const icons_1 = require("../../config/icons");
const placement = ['bottom-start', 'top-start', 'bottom-end', 'top-end'];
const ToolbarLink = ({ className, focus, onClick, active, enable, formInitialText, formReadOnlyText, onSubmit, removeLink, }) => {
    var _a;
    const buttonRef = react_1.default.useRef(null);
    const [open, , hide, toggleOpen] = (0, hooks_1.useBooleanState)(false);
    const popupOpen = enable && open;
    const shouldForceHide = open && !popupOpen;
    react_1.default.useLayoutEffect(() => {
        if (shouldForceHide) {
            hide();
        }
    }, [hide, shouldForceHide]);
    const handleClick = () => {
        if (active) {
            focus();
            removeLink();
            if (popupOpen) {
                hide();
            }
            return;
        }
        toggleOpen();
    };
    const handleSubmit = react_1.default.useCallback(({ url, text }) => {
        hide();
        focus();
        onSubmit(url, text);
        onClick === null || onClick === void 0 ? void 0 : onClick('addLink');
    }, [focus, hide, onClick, onSubmit]);
    return (react_1.default.createElement(react_1.default.Fragment, null,
        react_1.default.createElement(uikit_1.Tooltip, { disabled: popupOpen, content: (0, menubar_1.i18n)('link'), openDelay: toolbar_1.ToolbarTooltipDelay.Open, closeDelay: toolbar_1.ToolbarTooltipDelay.Close },
            react_1.default.createElement(uikit_1.Button, { size: "m", ref: buttonRef, view: active || popupOpen ? 'normal' : 'flat', selected: active, disabled: !enable, className: className, onClick: handleClick },
                react_1.default.createElement(uikit_1.Icon, { data: icons_1.icons.link.data, size: (_a = icons_1.icons.link.size) !== null && _a !== void 0 ? _a : 16 }))),
        react_1.default.createElement(uikit_1.Popup, { anchorRef: buttonRef, open: popupOpen, onClose: hide, placement: placement },
            react_1.default.createElement(LinkForm_1.LinkForm, { autoFocus: true, onCancel: hide, onSubmit: handleSubmit, initialText: formInitialText, readOnlyText: formReadOnlyText }))));
};
exports.ToolbarLink = ToolbarLink;
