"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MToolbarImage = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importDefault(require("react"));
const is_number_1 = tslib_1.__importDefault(require("is-number"));
const commands_1 = require("../../../markup/commands");
const const_1 = require("../../cm-upload/const");
const utils_1 = require("../../cm-upload/utils");
const ToolbarImage_1 = require("../custom/ToolbarImage");
const context_1 = require("./context");
const noop = (err) => {
    console.error(err);
};
const MToolbarImage = ({ focus, onClick, editor, className, }) => {
    const { uploadHandler, needToSetDimensionsForUploadedImages } = (0, context_1.useMarkupToolbarContext)();
    return (react_1.default.createElement(ToolbarImage_1.ToolbarImage, { focus: focus, onClick: onClick, className: className, active: false, enable: true, onSubmit: ({ url, name, alt, width, height }) => {
            (0, commands_1.insertImages)(editor.cm, [
                {
                    url,
                    alt,
                    title: name,
                    width: (0, is_number_1.default)(width) ? String(width) : '',
                    height: (0, is_number_1.default)(height) ? String(height) : '',
                },
            ]);
        }, uploadImages: uploadHandler, onSuccessUpload: async (res) => {
            const images = await toImageItems(res.success, Boolean(needToSetDimensionsForUploadedImages));
            (0, commands_1.insertImages)(editor.cm, images);
        } }));
};
exports.MToolbarImage = MToolbarImage;
async function toImageItems(items, withDimmensions) {
    const imgItems = [];
    await Promise.all(items.map(({ result, file }) => {
        var _a;
        const imgItem = { url: result.url, alt: (_a = result.name) !== null && _a !== void 0 ? _a : file.name };
        imgItems.push(imgItem);
        if (withDimmensions) {
            return (0, utils_1.getImageDimensions)(file).then(({ height }) => {
                imgItem.height = String(Math.min(height, const_1.IMG_MAX_HEIGHT));
            }, noop);
        }
        return Promise.resolve();
    }));
    return imgItems;
}
