"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useYfmEditor = void 0;
const react_1 = require("react");
const extensions_1 = require("../extensions");
const logger_1 = require("../logger");
const Editor_1 = require("./Editor");
const wysiwyg_preset_1 = require("./wysiwyg-preset");
function useYfmEditor(props, deps = []) {
    const editor = (0, react_1.useMemo)(() => {
        const renderStorage = new extensions_1.ReactRenderStorage();
        const extensions = (builder) => {
            builder.use(wysiwyg_preset_1.BundlePreset, Object.assign(Object.assign({}, props.extensionOptions), { reactRenderer: renderStorage, onCancel: () => {
                    editor.emit('cancel', null);
                    return true;
                }, onSubmit: () => {
                    editor.emit('submit', null);
                    return true;
                }, mdBreaks: props.breaks, fileUploadHandler: props.fileUploadHandler, needToSetDimensionsForUploadedImages: props.needToSetDimensionsForUploadedImages }));
            if (props.extraExtensions) {
                builder.use(props.extraExtensions, props.extensionOptions);
            }
        };
        return new Editor_1.EditorImpl(Object.assign(Object.assign({}, props), { extensions, renderStorage }));
    }, deps.concat(props.allowHTML, props.linkify, props.linkifyTlds, props.breaks, props.splitMode, props.needToSetDimensionsForUploadedImages));
    (0, react_1.useLayoutEffect)(() => {
        function onToolbarAction({ editorType, id }) {
            logger_1.logger.action({ mode: editorType, source: 'toolbar', action: id });
        }
        editor.on('toolbar-action', onToolbarAction);
        return () => {
            editor.off('toolbar-action', onToolbarAction);
            editor.destroy();
        };
    }, [editor]);
    return editor;
}
exports.useYfmEditor = useYfmEditor;
