"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BundlePreset = void 0;
const extensions_1 = require("../extensions");
const EditorModeKeymap_1 = require("../extensions/behavior/EditorModeKeymap");
const specs_1 = require("../extensions/specs");
const placeholder_1 = require("../i18n/placeholder");
const shortcuts_1 = require("../shortcuts");
const wysiwyg_1 = require("./config/wysiwyg");
const emoji_1 = require("./emoji");
const BundlePreset = (builder, opts) => {
    var _a, _b;
    const dropCursor = {
        color: 'var(--g-color-line-brand)',
        width: 2,
    };
    const options = Object.assign(Object.assign({}, opts), { cursor: { dropOptions: dropCursor }, clipboard: Object.assign({ pasteFileHandler: opts.fileUploadHandler }, opts.clipboard), selectionContext: Object.assign({ config: wysiwyg_1.wSelectionMenuConfig }, opts.selectionContext), commandMenu: Object.assign({ actions: wysiwyg_1.wCommandMenuConfig }, opts.commandMenu), baseSchema: Object.assign({ paragraphKey: shortcuts_1.formatter.toPM(shortcuts_1.Action.Text), paragraphPlaceholder: (node, parent) => {
                const isDocEmpty = !node.text && (parent === null || parent === void 0 ? void 0 : parent.type.name) === specs_1.BaseNode.Doc && parent.childCount === 1;
                return isDocEmpty ? (0, placeholder_1.i18n)('doc_empty') : null;
            } }, opts.baseSchema), breaks: Object.assign({ preferredBreak: (opts.mdBreaks ? 'soft' : 'hard') }, opts.breaks), history: Object.assign({ undoKey: shortcuts_1.formatter.toPM(shortcuts_1.Action.Undo), redoKey: shortcuts_1.formatter.toPM(shortcuts_1.Action.Redo) }, opts.history), bold: Object.assign({ boldKey: shortcuts_1.formatter.toPM(shortcuts_1.Action.Bold) }, opts.bold), italic: Object.assign({ italicKey: shortcuts_1.formatter.toPM(shortcuts_1.Action.Italic) }, opts.italic), strike: Object.assign({ strikeKey: shortcuts_1.formatter.toPM(shortcuts_1.Action.Strike) }, opts.strike), underline: Object.assign({ underlineKey: shortcuts_1.formatter.toPM(shortcuts_1.Action.Underline) }, opts.underline), code: Object.assign({ codeKey: shortcuts_1.formatter.toPM(shortcuts_1.Action.Code) }, opts.code), codeBlock: Object.assign({ codeBlockKey: shortcuts_1.formatter.toPM(shortcuts_1.Action.CodeBlock) }, opts.codeBlock), blockquote: Object.assign({ qouteKey: shortcuts_1.formatter.toPM(shortcuts_1.Action.Quote) }, opts.blockquote), link: Object.assign({ linkKey: shortcuts_1.formatter.toPM(shortcuts_1.Action.Link) }, opts.link), lists: Object.assign({ ulKey: shortcuts_1.formatter.toPM(shortcuts_1.Action.BulletList), olKey: shortcuts_1.formatter.toPM(shortcuts_1.Action.OrderedList), ulInputRules: { plus: false } }, opts.lists), imgSize: Object.assign({ imageUploadHandler: opts.fileUploadHandler, needToSetDimensionsForUploadedImages: opts.needToSetDimensionsForUploadedImages }, opts.imgSize), checkbox: Object.assign({ checkboxLabelPlaceholder: () => (0, placeholder_1.i18n)('checkbox') }, opts.checkbox), deflist: {
            deflistTermPlaceholder: () => (0, placeholder_1.i18n)('deflist_term'),
            deflistDescPlaceholder: () => (0, placeholder_1.i18n)('deflist_desc'),
        }, yfmCut: Object.assign({ yfmCutKey: shortcuts_1.formatter.toPM(shortcuts_1.Action.Cut), yfmCutTitlePlaceholder: () => (0, placeholder_1.i18n)('cut_title'), yfmCutContentPlaceholder: () => (0, placeholder_1.i18n)('cut_content') }, opts.yfmCut), yfmNote: Object.assign({ yfmNoteKey: shortcuts_1.formatter.toPM(shortcuts_1.Action.Note), yfmNoteTitlePlaceholder: () => (0, placeholder_1.i18n)('note_title') }, opts.yfmNote), yfmTable: Object.assign({ yfmTableCellPlaceholder: () => (0, placeholder_1.i18n)('table_cell') }, opts.yfmTable), yfmFile: Object.assign({ fileUploadHandler: opts.fileUploadHandler, needToSetDimensionsForUploadedImages: opts.needToSetDimensionsForUploadedImages }, opts.yfmFile), yfmHeading: Object.assign({ h1Key: shortcuts_1.formatter.toPM(shortcuts_1.Action.Heading1), h2Key: shortcuts_1.formatter.toPM(shortcuts_1.Action.Heading2), h3Key: shortcuts_1.formatter.toPM(shortcuts_1.Action.Heading3), h4Key: shortcuts_1.formatter.toPM(shortcuts_1.Action.Heading4), h5Key: shortcuts_1.formatter.toPM(shortcuts_1.Action.Heading5), h6Key: shortcuts_1.formatter.toPM(shortcuts_1.Action.Heading6), headingPlaceholder: (node) => `${(0, placeholder_1.i18n)('heading')} ${node.attrs[specs_1.YfmHeadingAttr.Level]}` }, opts.yfmHeading), emoji: Object.assign({ defs: emoji_1.emojiDefs }, opts.emoji), placeholder: {
            [specs_1.YfmNoteNode.NoteContent]: () => (0, placeholder_1.i18n)('note_content'),
        } });
    builder
        .use(extensions_1.BehaviorPreset, options)
        .use(extensions_1.BasePreset, options)
        .use(extensions_1.MarkdownMarksPreset, options)
        .use(extensions_1.MarkdownBlocksPreset, Object.assign(Object.assign({}, options), { image: false, heading: false }))
        .use(extensions_1.YfmPreset, options);
    const ignoreActions = [
        shortcuts_1.Action.Undo,
        shortcuts_1.Action.Redo,
        shortcuts_1.Action.Bold,
        shortcuts_1.Action.Italic,
        shortcuts_1.Action.Underline,
        shortcuts_1.Action.Strike,
        shortcuts_1.Action.Code,
        shortcuts_1.Action.Link,
        shortcuts_1.Action.Text,
        shortcuts_1.Action.Heading1,
        shortcuts_1.Action.Heading2,
        shortcuts_1.Action.Heading3,
        shortcuts_1.Action.Heading4,
        shortcuts_1.Action.Heading5,
        shortcuts_1.Action.Heading6,
        shortcuts_1.Action.BulletList,
        shortcuts_1.Action.OrderedList,
        shortcuts_1.Action.Quote,
        shortcuts_1.Action.CodeBlock,
        shortcuts_1.Action.Note,
        shortcuts_1.Action.Cut,
    ];
    const ignoreKeysList = (_b = (_a = opts.ignoreKeysList) === null || _a === void 0 ? void 0 : _a.slice()) !== null && _b !== void 0 ? _b : [];
    for (const action of ignoreActions) {
        const key = shortcuts_1.formatter.toPM(action);
        if (key)
            ignoreKeysList.push(key);
    }
    builder.use(EditorModeKeymap_1.EditorModeKeymap, {
        onSubmit: opts.onSubmit,
        onCancel: opts.onCancel,
        ignoreKeysList,
    });
};
exports.BundlePreset = BundlePreset;
