"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CodeMirrorViewExtension = void 0;
const prosemirror_keymap_1 = require("prosemirror-keymap");
const prosemirror_state_1 = require("prosemirror-state");
const lodash_1 = require("../../../lodash");
const TooltipPlugin_1 = require("./TooltipPlugin");
const YCThemeWatcher_1 = require("./YCThemeWatcher");
const CodeBlockView_1 = require("./cm/CodeBlockView");
const codemodes_1 = require("./cm/codemodes");
const commands_1 = require("./commands");
const const_1 = require("./const");
const CodeMirrorViewExtension = (builder, options) => {
    builder.addPlugin(() => {
        const watcherEmitter = new YCThemeWatcher_1.YCThemeStore();
        const langItems = (0, codemodes_1.getUniqueModeNames)().map((name) => ({
            value: name,
            title: (0, lodash_1.capitalize)(name),
        }));
        return [
            (0, YCThemeWatcher_1.ycThemeWatcherPlugin)(watcherEmitter),
            new prosemirror_state_1.Plugin({
                props: {
                    nodeViews: {
                        [const_1.codeBlockNodeName]: CodeBlockView_1.CodeBlockView.creator(Object.assign(Object.assign({}, options), { themeWatcher: watcherEmitter })),
                    },
                    // same as keymap({})
                    handleKeyDown: (0, prosemirror_keymap_1.keydownHandler)({
                        ArrowLeft: (0, commands_1.arrowHandler)('left', const_1.codeBlockNodeName),
                        ArrowRight: (0, commands_1.arrowHandler)('right', const_1.codeBlockNodeName),
                        ArrowUp: (0, commands_1.arrowHandler)('up', const_1.codeBlockNodeName),
                        ArrowDown: (0, commands_1.arrowHandler)('down', const_1.codeBlockNodeName),
                    }),
                },
                view: (view) => (0, TooltipPlugin_1.codeLangSelectTooltipViewCreator)(view, langItems),
            }),
        ];
    });
};
exports.CodeMirrorViewExtension = CodeMirrorViewExtension;
