"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.setCellRightAlign = exports.setCellCenterAlign = exports.setCellLeftAlign = void 0;
const markdown_1 = require("../../../../extensions/markdown");
exports.setCellLeftAlign = factory(markdown_1.TableCellAlign.Left);
exports.setCellCenterAlign = factory(markdown_1.TableCellAlign.Center);
exports.setCellRightAlign = factory(markdown_1.TableCellAlign.Right);
function factory(align) {
    return {
        isActive(state) {
            const cell = markdown_1.TableHelpers.findParentCell(state);
            return (cell === null || cell === void 0 ? void 0 : cell.node.attrs[markdown_1.TableAttrs.CellAlign]) === align;
        },
        isEnable(state) {
            return markdown_1.TableHelpers.isIntoTable(state);
        },
        run(state, dispatch) {
            const table = markdown_1.TableHelpers.findParentTable(state);
            const row = markdown_1.TableHelpers.findParentRow(state);
            const cell = markdown_1.TableHelpers.findParentCell(state);
            if (!cell || !row || !table) {
                return;
            }
            let cellIndex = -1;
            row.node.forEach((node, _, index) => {
                if (node === cell.node) {
                    cellIndex = index;
                }
            });
            if (cellIndex < 0) {
                return;
            }
            const tr = state.tr;
            const allRows = markdown_1.TableHelpers.findTableRows(table.node, state.schema);
            for (const row of allRows) {
                const rowCells = markdown_1.TableHelpers.findTableCells(row.node, state.schema);
                const cell = rowCells[cellIndex];
                // mmm, magic numbers ^_^
                const pos = table.pos + row.pos + cell.pos + 2;
                tr.setNodeMarkup(pos, undefined, Object.assign(Object.assign({}, cell.node.attrs), { [markdown_1.TableAttrs.CellAlign]: align }));
            }
            dispatch(tr);
        },
    };
}
