"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.tableControlsPlugin = exports.tableControlsPluginKey = void 0;
const prosemirror_state_1 = require("prosemirror-state");
const prosemirror_utils_1 = require("prosemirror-utils");
const prosemirror_view_1 = require("prosemirror-view");
const lodash_1 = require("../../../../lodash");
const table_utils_1 = require("../../../../table-utils");
const const_1 = require("../const");
const view_1 = require("./view");
const yfmTableCellView_1 = require("./yfmTableCellView");
exports.tableControlsPluginKey = new prosemirror_state_1.PluginKey('TableControlsPlugin');
function shouldUpdateState(prev, curr) {
    if (prev === null && curr === null)
        return false;
    if (prev === null || curr === null)
        return true;
    if (prev.left.from === curr.left.from &&
        prev.left.to === curr.left.to &&
        prev.upper.from === curr.upper.from &&
        prev.upper.to === curr.upper.to)
        return false;
    return true;
}
const tableControlsPlugin = () => new prosemirror_state_1.Plugin({
    key: exports.tableControlsPluginKey,
    state: {
        init: () => null,
        apply: (tr, prev) => {
            const meta = tr.getMeta(exports.tableControlsPluginKey);
            return meta === undefined ? prev : meta;
        },
    },
    props: {
        nodeViews: {
            [const_1.YfmTableNode.Table]: view_1.yfmTableView,
            [const_1.YfmTableNode.Cell]: yfmTableCellView_1.yfmTableCellView,
        },
        decorations(state) {
            const pluginState = exports.tableControlsPluginKey.getState(state);
            const parent = (0, prosemirror_utils_1.findParentNode)(table_utils_1.isTableNode)(state.selection) ||
                (0, prosemirror_utils_1.findSelectedNodeOfType)(state.schema.nodes.yfm_table)(state.selection);
            if (!parent)
                return null;
            const decs = [
                prosemirror_view_1.Decoration.node(parent.pos, parent.pos + parent.node.nodeSize, {
                    class: (0, view_1.viewB)('buttons-visible'),
                }),
            ];
            if (pluginState) {
                const { upper, left } = pluginState;
                decs.unshift(prosemirror_view_1.Decoration.node(left.from, left.to, {
                    class: (0, yfmTableCellView_1.yfmTableCellCn)('left-visible'),
                }), prosemirror_view_1.Decoration.node(upper.from, upper.to, {
                    class: (0, yfmTableCellView_1.yfmTableCellCn)('upper-visible'),
                }));
            }
            return prosemirror_view_1.DecorationSet.create(state.doc, decs);
        },
        handleDOMEvents: {
            mousemove: (0, lodash_1.throttle)((view, e) => {
                const prevState = exports.tableControlsPluginKey.getState(view.state);
                const pos = view.posAtCoords({ left: e.x, top: e.y });
                if (!pos)
                    return false;
                const cell = (0, prosemirror_utils_1.findParentNodeClosestToPos)(view.state.doc.resolve(pos.pos), table_utils_1.isTableCellNode);
                const table = (0, prosemirror_utils_1.findParentNodeClosestToPos)(view.state.doc.resolve(pos.pos), table_utils_1.isTableBodyNode);
                let state = null;
                if (cell && table) {
                    const contentDom = view.domAtPos(cell.pos + 1).node;
                    const rowNumber = contentDom.getAttribute('data-row-number');
                    const colNumber = contentDom.getAttribute('data-col-number');
                    const rows = (0, prosemirror_utils_1.findChildren)(table.node, table_utils_1.isTableRowNode);
                    const parentRow = rows[Number(rowNumber)];
                    const firstRow = rows[0];
                    const parentCol = (0, prosemirror_utils_1.findChildren)(rows[0].node, table_utils_1.isTableCellNode)[Number(colNumber)];
                    const rowFrom = parentRow.pos + table.start;
                    const rowTo = parentRow.pos + parentRow.node.nodeSize + table.start;
                    const firstRowFrom = firstRow.pos + table.start;
                    state = {
                        upper: {
                            from: parentCol.pos + firstRowFrom + 1,
                            to: parentCol.pos + firstRowFrom + 1 + parentCol.node.nodeSize,
                        },
                        left: {
                            from: rowFrom,
                            to: rowTo,
                        },
                    };
                }
                if (shouldUpdateState(prevState, state)) {
                    view.dispatch(view.state.tr.setMeta(exports.tableControlsPluginKey, state));
                }
                return false;
            }, 100),
        },
    },
});
exports.tableControlsPlugin = tableControlsPlugin;
