"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.toTabs = exports.toHr = exports.toMermaid = exports.insertFiles = exports.insertIframe = exports.insertImages = exports.insertAnchor = exports.insertLink = exports.insertTable = exports.wrapToYfmLayout = exports.wrapToYfmBlock = exports.wrapToNote = exports.wrapToCut = exports.liftListItem = exports.sinkListItem = exports.wrapToMathBlock = exports.wrapToMathInline = exports.toCodeBlock = exports.toInlineCode = exports.toOrderedList = exports.toBulletList = exports.toQuote = exports.toCheckbox = exports.colorify = exports.toH6 = exports.toH5 = exports.toH4 = exports.toH3 = exports.toH2 = exports.toH1 = exports.wrapToMark = exports.wrapToMonospace = exports.wrapToUnderline = exports.wrapToStrike = exports.wrapToItalic = exports.wrapToBold = void 0;
exports.wrapToBold = wrapTo('**');
exports.wrapToItalic = wrapTo('*');
exports.wrapToStrike = wrapTo('~~');
exports.wrapToUnderline = wrapTo('++');
exports.wrapToMonospace = wrapTo('##');
exports.wrapToMark = wrapTo('==');
exports.toH1 = toHeading(1);
exports.toH2 = toHeading(2);
exports.toH3 = toHeading(3);
exports.toH4 = toHeading(4);
exports.toH5 = toHeading(5);
exports.toH6 = toHeading(6);
const colorify = (cm, color) => wrapTo(`{${color}}(`, ')')(cm);
exports.colorify = colorify;
const wrapPerLine = ({ beforeText, afterText = beforeText, wrapEmptyLine = false }) => (cm) => {
    const from = cm.getCursor('from');
    const to = cm.getCursor('to');
    const lines = [];
    for (let line = from.line; line <= to.line; line++) {
        lines.push(cm.getLine(line));
    }
    lines.forEach((l, i) => {
        const currLine = from.line + i;
        const shouldAppend = l.length || (wrapEmptyLine && !l.length) || (l.length === 0 && lines.length === 1);
        cm.replaceRange(shouldAppend ? beforeText + l + afterText : l, { line: currLine, ch: 0 }, { line: currLine, ch: Number.MAX_SAFE_INTEGER });
    });
    cm.setSelection({ line: to.line, ch: Number.MAX_SAFE_INTEGER });
};
const needEmptyLines = (cm) => {
    var _a, _b;
    const start = cm.getCursor('from');
    const end = cm.getCursor('to');
    const emptyLineBefore = ((_a = cm.getLine(end.line - 1)) === null || _a === void 0 ? void 0 : _a.length) || start.ch !== 0 ? '\n\n' : '';
    const emptyLineAfter = ((_b = cm.getLine(end.line + 1)) === null || _b === void 0 ? void 0 : _b.length) || end.ch !== cm.getLine(end.line).length ? '\n\n' : '';
    return { emptyLineBefore, emptyLineAfter };
};
const wrapToBlock = (openToken, closeToken = openToken) => (cm) => {
    const text = cm.getSelection();
    const start = cm.getCursor('from');
    const { emptyLineBefore, emptyLineAfter } = needEmptyLines(cm);
    cm.replaceSelection(emptyLineBefore + openToken + text + closeToken + emptyLineAfter);
    cm.setSelection({
        line: start.line + (emptyLineBefore + openToken + text).split('\n').length - 1,
        ch: Number.MAX_SAFE_INTEGER,
    });
};
exports.toCheckbox = wrapPerLine({ beforeText: '[ ] ', afterText: '' });
exports.toQuote = wrapPerLine({ beforeText: '> ', afterText: '', wrapEmptyLine: true });
exports.toBulletList = wrapPerLine({ beforeText: '* ', afterText: '' });
exports.toOrderedList = wrapPerLine({ beforeText: '1. ', afterText: '' });
exports.toInlineCode = wrapTo('`');
exports.toCodeBlock = wrapToBlock('```\n', '\n```');
exports.wrapToMathInline = wrapTo('$');
exports.wrapToMathBlock = wrapToBlock('$$\n', '\n$$');
exports.sinkListItem = wrapPerLine({ beforeText: '  ', afterText: '' });
const liftListItem = (cm) => {
    const from = cm.getCursor('from');
    const to = cm.getCursor('to');
    const lines = [];
    for (let line = from.line; line <= to.line; line++) {
        lines.push(cm.getLine(line));
    }
    lines.forEach((l, i) => {
        const currLine = from.line + i;
        cm.replaceRange(l.replace(/ {2}/, ''), { line: currLine, ch: 0 }, { line: currLine, ch: Number.MAX_SAFE_INTEGER });
    });
};
exports.liftListItem = liftListItem;
exports.wrapToCut = wrapToBlock('{% cut "title" %}\n\n', '\n\n{% endcut %}');
exports.wrapToNote = wrapToBlock('{% note info %}\n\n', '\n\n{% endnote %}');
exports.wrapToYfmBlock = wrapToBlock('{% block %}\n\n', '\n\n{% endblock %}');
exports.wrapToYfmLayout = wrapToBlock('{% layout gap=l %}\n\n{% block %}\n\n', '\n\n{% endblock %}\n\n{% endlayout %}');
const insertTable = (cm) => {
    const tableTokens = ['#|', '||', '', '|', '', '||', '||', '', '|', '', '||', '|#', ''].join('\n\n');
    const { emptyLineBefore, emptyLineAfter } = needEmptyLines(cm);
    cm.replaceSelection(emptyLineBefore + tableTokens + emptyLineAfter);
};
exports.insertTable = insertTable;
const insertLink = ({ url, text, title }) => (cm) => {
    if (cm.getSelection()) {
        // ignore text if editor has selection
        wrapTo('[', `](${url}${title ? ` "${title}"` : ''})`)(cm);
        return;
    }
    const markup = `[${text !== null && text !== void 0 ? text : ''}](${url}${title ? ` "${title}"` : ''})`;
    cm.replaceSelection(markup);
};
exports.insertLink = insertLink;
const insertAnchor = ({ href: href, text, title } = {}) => (cm) => {
    if (cm.getSelection()) {
        // ignore text if editor has selection
        wrapTo('#[', `](${href !== null && href !== void 0 ? href : ''}${title ? ` "${title}"` : ''})`)(cm);
        return;
    }
    const markup = `#[${text !== null && text !== void 0 ? text : ''}](${href !== null && href !== void 0 ? href : ''}${title ? ` "${title}"` : ''})`;
    cm.replaceSelection(markup);
};
exports.insertAnchor = insertAnchor;
function insertImages(cm, images) {
    const markup = images
        .map(({ title, url, alt, width, height }) => {
        const titleStr = title ? ` "${title}"` : '';
        const sizeStr = (width !== null && width !== void 0 ? width : height) ? ` =${width !== null && width !== void 0 ? width : ''}x${height !== null && height !== void 0 ? height : ''}` : '';
        return `![${alt !== null && alt !== void 0 ? alt : ''}](${url !== null && url !== void 0 ? url : ''}${titleStr}${sizeStr})`;
    })
        .join(' ');
    cm.replaceSelection(markup);
}
exports.insertImages = insertImages;
const insertIframe = (cm, args) => {
    const argsMarkup = Object.entries(args)
        .map(([k, v]) => `${k}=${v}`)
        .join(' ');
    const markup = `/iframe/(${argsMarkup})`;
    const { emptyLineBefore, emptyLineAfter } = needEmptyLines(cm);
    cm.replaceSelection(emptyLineBefore + markup + emptyLineAfter);
};
exports.insertIframe = insertIframe;
const insertFiles = (cm, files) => {
    const markup = files
        .map((attrs) => {
        const attrsStr = Object.entries(attrs)
            .map(([key, value]) => `${key}="${value.replace('"', '')}"`)
            .join(' ');
        return `{% file ${attrsStr} %}`;
    })
        .join(' ');
    cm.replaceSelection(markup);
};
exports.insertFiles = insertFiles;
const toMermaid = (cm) => {
    cm.replaceSelection(`\`\`\`mermaid
sequenceDiagram
	Alice->>Bob: Hi Bob
	Bob->>Alice: Hi Alice
\`\`\``);
};
exports.toMermaid = toMermaid;
function wrapTo(beforeText, afterText = beforeText) {
    return function (cm) {
        const text = cm.getSelection();
        const start = cm.getCursor('from');
        const end = cm.getCursor('to');
        cm.replaceSelection(beforeText + text + afterText);
        cm.setSelection({ line: start.line, ch: start.ch + beforeText.length }, { line: end.line, ch: start.line === end.line ? end.ch + beforeText.length : end.ch });
    };
}
function toHeading(level) {
    const re = /^\s*#*\s*/;
    const str = Array(level).fill('#').join('') + ' ';
    return function (cm) {
        const startLine = cm.getCursor('from').line;
        const endLine = cm.getCursor('to').line;
        if (startLine === endLine) {
            replace(startLine, cm.getLine(startLine));
            return;
        }
        for (let i = startLine; i <= endLine; i++) {
            const lineContent = cm.getLine(i);
            if (lineContent.trim().length) {
                replace(i, lineContent);
            }
        }
        function replace(line, content) {
            const newContent = content.replace(re, str);
            cm.replaceRange(newContent, { line, ch: 0 }, { line, ch: Number.MAX_SAFE_INTEGER });
        }
    };
}
function toHr(cm) {
    const start = cm.getCursor('from');
    cm.replaceSelection('---\n');
    cm.setSelection({ line: start.line + 1, ch: 0 });
}
exports.toHr = toHr;
function toTabs(cm) {
    cm.replaceSelection(['{% list tabs %}', '', '- ', '  ', '', '{% endlist %}', ''].join('\n'));
}
exports.toTabs = toTabs;
