import type { ReactNode } from 'react';
import { CommonEditor, MarkupString } from '../common';
import { YfmEditorOptions as WysiwygOptions } from '../core';
import { ReactRenderStorage } from '../extensions';
import { Receiver } from '../utils/event-emitter';
import type { FileUploadHandler } from '../utils/upload';
export declare type EditorType = 'wysiwyg' | 'markup';
export declare type SplitMode = false | 'horizontal' | 'vertical';
export declare type RenderPreview = ({ getValue, mode, }: {
    getValue: () => MarkupString;
    mode: 'preview' | 'split';
}) => ReactNode;
interface EventMap {
    change: null;
    cancel: null;
    submit: null;
    'toolbar-action': {
        editorType: EditorType;
        id: string;
        attrs?: {
            [key: string]: any;
        };
    };
    'change-editor-type': {
        type: EditorType;
    };
    'change-toolbar-visibility': {
        visible: boolean;
    };
    'change-split-mode-enabled': {
        splitModeEnabled: boolean;
    };
}
export interface Editor extends Receiver<EventMap>, CommonEditor {
    readonly currentType: EditorType;
    readonly toolbarVisible: boolean;
    setEditorType(type: EditorType): void;
    moveCursor(position: 'start' | 'end' | {
        line: number;
    }): void;
}
export declare type EditorOptions = Pick<WysiwygOptions, 'allowHTML' | 'linkify' | 'linkifyTlds' | 'attrs' | 'extensions'> & {
    initialMarkup?: MarkupString;
    /** @default 'wysiwyg' */
    initialEditorType?: EditorType;
    /** @default true */
    initialToolbarVisible?: boolean;
    /** @default false
     * Has no effect if splitMode is false or undefined
     */
    initialSplitModeEnabled?: boolean;
    renderStorage: ReactRenderStorage;
    fileUploadHandler?: FileUploadHandler;
    /**
     * If we need to set dimensions for uploaded images
     *
     * @default false
     */
    needToSetDimensionsForUploadedImages?: boolean;
    /**
     * Called before switching from the markup editor to the wysiwyg editor.
     * You can use it to pre-process the value from the markup editor before it gets into the wysiwyg editor.
     */
    prepareRawMarkup?: (value: MarkupString) => MarkupString;
    splitMode?: SplitMode;
    renderPreview?: RenderPreview;
};
export {};
