import React from 'react';
import { ClassNameProps } from '../classname';
import { MarkupString } from '../common';
import { Extension } from '../core';
import type { Editor, RenderPreview, SplitMode, EditorType as YfmEditorType } from './Editor';
import { YfmEditorViewProps } from './YfmEditorView';
import type { MToolbarData } from './config/markup';
import type { WToolbarData } from './config/wysiwyg';
import { UseYfmEditorProps } from './useYfmEditor';
export type { ExtensionsOptions } from './wysiwyg-preset';
export type { EditorType as YfmEditorType } from './Editor';
/**
 * @deprecated
 * The `YfmEditor` component will be removed soon.
 * Use `useYfmEditor` hook and `YfmEditorView` component instead.
 */
export declare type YfmEditorRef = Editor & {
    domElem: () => HTMLElement | null;
};
/**
 * @deprecated
 * The `YfmEditor` component will be removed soon.
 * Use `useYfmEditor` hook and `YfmEditorView` component instead.
 */
export declare type YfmEditorProps<T extends object = {}> = ClassNameProps & Pick<YfmEditorViewProps, 'settingsVisible' | 'toaster' | 'wysiwygHiddenActionsConfig' | 'markupHiddenActionsConfig'> & {
    /** @default false'
     * Split mode orientation
     * Has no effect if renderPreview is not defined
     */
    splitMode?: SplitMode;
    /** @default true
     *  Is toolbar sticky
     */
    stickyToolbar?: boolean;
    autofocus?: boolean;
    /** initial yfm markup
     * Used only on first render
     */
    initialContent?: MarkupString;
    /** @default 'wysiwyg'
     * Used only on first render
     */
    initialEditorType?: YfmEditorType;
    /** @default true
     * Used only on first render
     */
    initialMenuVisible?: boolean;
    /** @default false
     * Used only on first render
     */
    initialSplitModeEnabled?: boolean;
    /** menu bar config for markup editor */
    markupMenubarData?: MToolbarData;
    /** menu bar config for wysiwyg editor */
    wysiwygMenubarData?: WToolbarData;
    /** Used only first value. Сhanging the value will not lead to anything */
    wysiwygExtraExtensions?: Extension;
    /** allowing HTML in markup */
    wysiwygAllowHTML?: boolean;
    /** markdown-it-attrs options */
    wysiwygAttrs?: UseYfmEditorProps['attrs'];
    /** allowing autoconvert URL-like text to links */
    wysiwygLinkify?: boolean;
    wysiwygLinkifyTlds?: string | string[];
    /**
     * Sets a soft break preferred for breaks inside a paragraph.
     *
     * If you set the "breaks:true" option for yfm-transform or markdrown-it,
     * set this param to "true" too, and the editor will add soft break
     * (a break without backslash) for breaks in the paragraph.
     *
     * Default: "true" – like in yfm-transform
     */
    wysiwygBreaks?: boolean;
    /** callback to action in menubar */
    onMenuBarAction?(data: {
        action: string;
        editorType: YfmEditorType;
    }): void;
    /** callback on change split mode */
    onSplitModeEnabledChange?(splitModeEnabled: boolean): void;
    /** function that will transform raw markup to html */
    renderPreview?: RenderPreview;
    /** callback on change editor type */
    onEditorTypeChange?(newType: YfmEditorType): void;
    /** callback on change menubar visibility */
    onMenuVisibleChange?(visible: boolean): void;
    /** callback on editors content change */
    onMarkupChange?(editor: YfmEditorRef, editorType: YfmEditorType): void;
    /**
     * Called before switching from the markup editor to the wysiwyg editor.
     * You can use it to pre-process the value from the markup editor before it gets into the wysiwyg editor.
     */
    prepareRawMarkup?(value: string): string;
    /** Submit handler. Return true to stop propagation. */
    onSubmit?(editor: YfmEditorRef): boolean;
    /** Cancel handler. Return true to stop propagation. */
    onCancel?(editor: YfmEditorRef): boolean;
    onFileUpload?: UseYfmEditorProps['fileUploadHandler'];
    extensionOptions?: UseYfmEditorProps<T>['extensionOptions'];
    /**
     * If we need to set dimensions for uploaded images
     *
     * @default false
     */
    needToSetDimensionsForUploadedImages?: boolean;
};
/**
 * @deprecated
 * The `YfmEditor` component will be removed soon.
 * Use `useYfmEditor` hook and `YfmEditorView` component instead.
 */
export declare const YfmEditor: <T extends object = {}>(props: ClassNameProps & Pick<YfmEditorViewProps, "settingsVisible" | "markupHiddenActionsConfig" | "wysiwygHiddenActionsConfig" | "toaster"> & {
    /** @default false'
     * Split mode orientation
     * Has no effect if renderPreview is not defined
     */
    splitMode?: SplitMode | undefined;
    /** @default true
     *  Is toolbar sticky
     */
    stickyToolbar?: boolean | undefined;
    autofocus?: boolean | undefined;
    /** initial yfm markup
     * Used only on first render
     */
    initialContent?: MarkupString | undefined;
    /** @default 'wysiwyg'
     * Used only on first render
     */
    initialEditorType?: YfmEditorType | undefined;
    /** @default true
     * Used only on first render
     */
    initialMenuVisible?: boolean | undefined;
    /** @default false
     * Used only on first render
     */
    initialSplitModeEnabled?: boolean | undefined;
    /** menu bar config for markup editor */
    markupMenubarData?: MToolbarData | undefined;
    /** menu bar config for wysiwyg editor */
    wysiwygMenubarData?: WToolbarData | undefined;
    /** Used only first value. Сhanging the value will not lead to anything */
    wysiwygExtraExtensions?: Extension | undefined;
    /** allowing HTML in markup */
    wysiwygAllowHTML?: boolean | undefined;
    /** markdown-it-attrs options */
    wysiwygAttrs?: UseYfmEditorProps['attrs'];
    /** allowing autoconvert URL-like text to links */
    wysiwygLinkify?: boolean | undefined;
    wysiwygLinkifyTlds?: string | string[] | undefined;
    /**
     * Sets a soft break preferred for breaks inside a paragraph.
     *
     * If you set the "breaks:true" option for yfm-transform or markdrown-it,
     * set this param to "true" too, and the editor will add soft break
     * (a break without backslash) for breaks in the paragraph.
     *
     * Default: "true" – like in yfm-transform
     */
    wysiwygBreaks?: boolean | undefined;
    /** callback to action in menubar */
    onMenuBarAction?(data: {
        action: string;
        editorType: YfmEditorType;
    }): void;
    /** callback on change split mode */
    onSplitModeEnabledChange?(splitModeEnabled: boolean): void;
    /** function that will transform raw markup to html */
    renderPreview?: RenderPreview | undefined;
    /** callback on change editor type */
    onEditorTypeChange?(newType: YfmEditorType): void;
    /** callback on change menubar visibility */
    onMenuVisibleChange?(visible: boolean): void;
    /** callback on editors content change */
    onMarkupChange?(editor: YfmEditorRef, editorType: YfmEditorType): void;
    /**
     * Called before switching from the markup editor to the wysiwyg editor.
     * You can use it to pre-process the value from the markup editor before it gets into the wysiwyg editor.
     */
    prepareRawMarkup?(value: string): string;
    /** Submit handler. Return true to stop propagation. */
    onSubmit?(editor: YfmEditorRef): boolean;
    /** Cancel handler. Return true to stop propagation. */
    onCancel?(editor: YfmEditorRef): boolean;
    onFileUpload?: UseYfmEditorProps['fileUploadHandler'];
    extensionOptions?: (Omit<import("./wysiwyg-preset").ExtensionsOptions, "reactRenderer"> & T) | undefined;
    /**
     * If we need to set dimensions for uploaded images
     *
     * @default false
     */
    needToSetDimensionsForUploadedImages?: boolean | undefined;
} & {
    ref?: React.Ref<YfmEditorRef> | undefined;
}) => JSX.Element;
