import React from 'react';
import { Button, Icon, Popup, Tooltip } from '@gravity-ui/uikit';
import { LinkForm } from '../../../forms/LinkForm';
import { i18n } from '../../../i18n/menubar';
import { useBooleanState } from '../../../react-utils/hooks';
import { ToolbarTooltipDelay } from '../../../toolbar';
import { icons } from '../../config/icons';
const placement = ['bottom-start', 'top-start', 'bottom-end', 'top-end'];
export const ToolbarLink = ({ className, focus, onClick, active, enable, formInitialText, formReadOnlyText, onSubmit, removeLink, }) => {
    var _a;
    const buttonRef = React.useRef(null);
    const [open, , hide, toggleOpen] = useBooleanState(false);
    const popupOpen = enable && open;
    const shouldForceHide = open && !popupOpen;
    React.useLayoutEffect(() => {
        if (shouldForceHide) {
            hide();
        }
    }, [hide, shouldForceHide]);
    const handleClick = () => {
        if (active) {
            focus();
            removeLink();
            if (popupOpen) {
                hide();
            }
            return;
        }
        toggleOpen();
    };
    const handleSubmit = React.useCallback(({ url, text }) => {
        hide();
        focus();
        onSubmit(url, text);
        onClick === null || onClick === void 0 ? void 0 : onClick('addLink');
    }, [focus, hide, onClick, onSubmit]);
    return (React.createElement(React.Fragment, null,
        React.createElement(Tooltip, { disabled: popupOpen, content: i18n('link'), openDelay: ToolbarTooltipDelay.Open, closeDelay: ToolbarTooltipDelay.Close },
            React.createElement(Button, { size: "m", ref: buttonRef, view: active || popupOpen ? 'normal' : 'flat', selected: active, disabled: !enable, className: className, onClick: handleClick },
                React.createElement(Icon, { data: icons.link.data, size: (_a = icons.link.size) !== null && _a !== void 0 ? _a : 16 }))),
        React.createElement(Popup, { anchorRef: buttonRef, open: popupOpen, onClose: hide, placement: placement },
            React.createElement(LinkForm, { autoFocus: true, onCancel: hide, onSubmit: handleSubmit, initialText: formInitialText, readOnlyText: formReadOnlyText }))));
};
